require 'gateway'
require 'cassandra-cql'

# Have to do this once so that ::CassandraCQL::Thrift::Client::TransportException
# is factoried out of thrift client factory... :(
ThriftClient.new CassandraCQL::Thrift::Client, []

module Gateway
  class CassandraCQL < Gateway::Base
    categorize_error [
      ::CassandraCQL::Thrift::Client::TransportException,
      ::CassandraCQL::Thrift::UnavailableException
    ], :as => :bad_gateway

    categorize_error [
      ::CassandraCQL::Thrift::TimedOutException
    ], :as => :timeout

    categorize_error [
      ::CassandraCQL::Thrift::Client::TransportException,
      ::CassandraCQL::Thrift::UnavailableException,
      ::CassandraCQL::Thrift::TimedOutException
    ], :as => :retry

    alias :__execute__ :execute

    def execute *args
      arguments = args.dup
      statement = args.shift
      block = args.pop if block_given?
      opts = args.last.is_a?(Hash) ? args.pop : {}
      bind_vars = args

      __execute__('cql', statement.to_s, opts) do |conn|
        if block
          conn.execute statement, *bind_vars, &block
        else
          conn.execute statement, *bind_vars
        end
      end
    end

    protected

    def connect
      opt = options[:keyspace] ? {:keyspace => options[:keyspace]} : {}
      ::CassandraCQL::Database.new options[:servers], opt
    end

    def disconnect(conn)
      conn.disconnect!
    end

    def reconnect(conn)
      conn.reset!
    end
  end
end
